/*
 * This code is taken from the official Window List Gnome Shell extension.
 *
 * Licensing information:
 * Window List Gnome Shell extension is distributed under the terms
 * of the GNU General Public License, version 2 or later.
 */

const Lang = imports.lang;
const Main = imports.ui.main;
const Clutter = imports.gi.Clutter;


const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const WindowList = Me.imports.windowList;

let appMenu;
let container;
let panel;
let windowList;
let panelConnectId;

function init() {
}

function enable() {
    panel = Main.panel;
    container = panel._leftBox;
    appMenu = panel.statusArea['appMenu'];

    panelConnectId = panel.actor.connect('allocate', allocate);
    container.remove_child(appMenu.container);

    windowList = new WindowList.WindowList();

    container.insert_child_at_index( windowList.actor, 2 );
}

function disable() {
    container.remove_child(windowList.actor);
    container.add_child(appMenu.container);
    windowList.actor.destroy();
    panel.actor.disconnect(panelConnectId);

    appMenu = null;
    container = null;
    panel = null;
    windowList = null;
    panelConnectId = null;
}

function allocate(actor, box, flags) {
    let allocWidth = box.x2 - box.x1;
    let allocHeight = box.y2 - box.y1;

    let [leftMinWidth, leftNaturalWidth] = panel._leftBox.get_preferred_width(-1);
    let [centerMinWidth, centerNaturalWidth] = panel._centerBox.get_preferred_width(-1);
    let [rightMinWidth, rightNaturalWidth] = panel._rightBox.get_preferred_width(-1);

    let sideWidth = allocWidth - rightNaturalWidth - centerNaturalWidth;

    let childBox = new Clutter.ActorBox();

    childBox.y1 = 0;
    childBox.y2 = allocHeight;
    if (panel.actor.get_text_direction() == Clutter.TextDirection.RTL) {
	      childBox.x1 = allocWidth - Math.min(Math.floor(sideWidth), leftNaturalWidth);
	      childBox.x2 = allocWidth;
    } else {
	      childBox.x1 = 0;
	      childBox.x2 = sideWidth;
    }
    panel._leftBox.allocate(childBox, flags);

    childBox.y1 = 0;
    childBox.y2 = allocHeight;
    if (panel.actor.get_text_direction() == Clutter.TextDirection.RTL) {
	      childBox.x1 = rightNaturalWidth;
	      childBox.x2 = childBox.x1 + centerNaturalWidth;
    } else {
	      childBox.x1 = allocWidth - centerNaturalWidth - rightNaturalWidth;
	      childBox.x2 = childBox.x1 + centerNaturalWidth;
    }
    panel._centerBox.allocate(childBox, flags);

    childBox.y1 = 0;
    childBox.y2 = allocHeight;
    if (panel.actor.get_text_direction() == Clutter.TextDirection.RTL) {
	      childBox.x1 = 0;
	      childBox.x2 = rightNaturalWidth;
    } else {
	      childBox.x1 = allocWidth - rightNaturalWidth;
	      childBox.x2 = allocWidth;
    }
    panel._rightBox.allocate(childBox, flags);

    let [cornerMinWidth, cornerWidth] = panel._leftCorner.actor.get_preferred_width(-1);
    let [cornerMinHeight, cornerHeight] = panel._leftCorner.actor.get_preferred_width(-1);
    childBox.x1 = 0;
    childBox.x2 = cornerWidth;
    childBox.y1 = allocHeight;
    childBox.y2 = allocHeight + cornerHeight;
    panel._leftCorner.actor.allocate(childBox, flags);

    let [cornerMinWidth, cornerWidth] = panel._rightCorner.actor.get_preferred_width(-1);
    let [cornerMinHeight, cornerHeight] = panel._rightCorner.actor.get_preferred_width(-1);
    childBox.x1 = allocWidth - cornerWidth;
    childBox.x2 = allocWidth;
    childBox.y1 = allocHeight;
    childBox.y2 = allocHeight + cornerHeight;
    panel._rightCorner.actor.allocate(childBox, flags);
}

function someWindowListContains(actor) {
      return windowList.actor.contains(actor);
}
